"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const constants_1 = require("../common/constants");
const constants_2 = require("../constants");
const decorators_1 = require("../common/decorators");
class RunController extends events_1.EventEmitter {
    constructor($analyticsService, $buildDataService, $buildController, $deviceDebugAppService, $deviceInstallAppService, $deviceRefreshAppService, $devicesService, $errors, $hmrStatusService, $hooksService, $liveSyncServiceResolver, $logger, $platformsDataService, $pluginsService, $prepareController, $prepareDataService, $prepareNativePlatformService, $projectDataService, $runEmitter) {
        super();
        this.$analyticsService = $analyticsService;
        this.$buildDataService = $buildDataService;
        this.$buildController = $buildController;
        this.$deviceDebugAppService = $deviceDebugAppService;
        this.$deviceInstallAppService = $deviceInstallAppService;
        this.$deviceRefreshAppService = $deviceRefreshAppService;
        this.$devicesService = $devicesService;
        this.$errors = $errors;
        this.$hmrStatusService = $hmrStatusService;
        this.$hooksService = $hooksService;
        this.$liveSyncServiceResolver = $liveSyncServiceResolver;
        this.$logger = $logger;
        this.$platformsDataService = $platformsDataService;
        this.$pluginsService = $pluginsService;
        this.$prepareController = $prepareController;
        this.$prepareDataService = $prepareDataService;
        this.$prepareNativePlatformService = $prepareNativePlatformService;
        this.$projectDataService = $projectDataService;
        this.$runEmitter = $runEmitter;
        this.processesInfo = {};
    }
    run(runData) {
        return __awaiter(this, void 0, void 0, function* () {
            const { projectDir, liveSyncInfo, deviceDescriptors } = runData;
            const projectData = this.$projectDataService.getProjectData(projectDir);
            yield this.initializeSetup(projectData);
            const platforms = this.$devicesService.getPlatformsFromDeviceDescriptors(deviceDescriptors);
            const deviceDescriptorsForInitialSync = this.getDeviceDescriptorsForInitialSync(projectDir, deviceDescriptors);
            this.persistData(projectDir, deviceDescriptors, platforms);
            const shouldStartWatcher = !liveSyncInfo.skipWatcher && !!this.processesInfo[projectDir].deviceDescriptors.length;
            if (shouldStartWatcher && liveSyncInfo.useHotModuleReload) {
                this.$hmrStatusService.attachToHmrStatusEvent();
            }
            this.$prepareController.on(constants_2.PREPARE_READY_EVENT_NAME, (data) => __awaiter(this, void 0, void 0, function* () {
                yield this.syncChangedDataOnDevices(data, projectData, liveSyncInfo, deviceDescriptors);
            }));
            yield this.syncInitialDataOnDevices(projectData, liveSyncInfo, deviceDescriptorsForInitialSync);
            this.attachDeviceLostHandler();
        });
    }
    stop(projectDir, deviceIdentifiers, stopOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const liveSyncProcessInfo = this.processesInfo[projectDir];
            if (liveSyncProcessInfo && !liveSyncProcessInfo.isStopped) {
                const shouldAwaitPendingOperation = !stopOptions || stopOptions.shouldAwaitAllActions;
                const deviceIdentifiersToRemove = deviceIdentifiers || _.map(liveSyncProcessInfo.deviceDescriptors, d => d.identifier);
                const removedDeviceIdentifiers = _.remove(liveSyncProcessInfo.deviceDescriptors, descriptor => _.includes(deviceIdentifiersToRemove, descriptor.identifier))
                    .map(descriptor => descriptor.identifier);
                _.each(liveSyncProcessInfo.platforms, platform => {
                    const devices = this.$devicesService.getDevicesForPlatform(platform);
                    if (!devices || !devices.length) {
                        this.$prepareController.stopWatchers(projectDir, platform);
                    }
                });
                if (!deviceIdentifiers || !deviceIdentifiers.length || !liveSyncProcessInfo.deviceDescriptors || !liveSyncProcessInfo.deviceDescriptors.length) {
                    if (liveSyncProcessInfo.timer) {
                        clearTimeout(liveSyncProcessInfo.timer);
                    }
                    _.each(liveSyncProcessInfo.platforms, platform => {
                        this.$prepareController.stopWatchers(projectDir, platform);
                    });
                    liveSyncProcessInfo.isStopped = true;
                    if (liveSyncProcessInfo.actionsChain && shouldAwaitPendingOperation) {
                        yield liveSyncProcessInfo.actionsChain;
                    }
                    liveSyncProcessInfo.deviceDescriptors = [];
                    const projectData = this.$projectDataService.getProjectData(projectDir);
                    yield this.$hooksService.executeAfterHooks('watch', {
                        hookArgs: {
                            projectData
                        }
                    });
                }
                else if (liveSyncProcessInfo.currentSyncAction && shouldAwaitPendingOperation) {
                    yield liveSyncProcessInfo.currentSyncAction;
                }
                _.each(removedDeviceIdentifiers, deviceIdentifier => {
                    this.$runEmitter.emitRunStoppedEvent(projectDir, deviceIdentifier);
                });
            }
        });
    }
    getDeviceDescriptors(projectDir) {
        const liveSyncProcessesInfo = this.processesInfo[projectDir] || {};
        const currentDescriptors = liveSyncProcessesInfo.deviceDescriptors;
        return currentDescriptors || [];
    }
    getDeviceDescriptorsForInitialSync(projectDir, deviceDescriptors) {
        const currentRunData = this.processesInfo[projectDir];
        const isAlreadyLiveSyncing = currentRunData && !currentRunData.isStopped;
        const deviceDescriptorsForInitialSync = isAlreadyLiveSyncing ? _.differenceBy(deviceDescriptors, currentRunData.deviceDescriptors, "identifier") : deviceDescriptors;
        return deviceDescriptorsForInitialSync;
    }
    initializeSetup(projectData) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.$pluginsService.ensureAllDependenciesAreInstalled(projectData);
            }
            catch (err) {
                this.$logger.trace(err);
                this.$errors.failWithoutHelp(`Unable to install dependencies. Make sure your package.json is valid and all dependencies are correct. Error is: ${err.message}`);
            }
        });
    }
    attachDeviceLostHandler() {
        this.$devicesService.on(constants_1.DeviceDiscoveryEventNames.DEVICE_LOST, (device) => __awaiter(this, void 0, void 0, function* () {
            this.$logger.trace(`Received ${constants_1.DeviceDiscoveryEventNames.DEVICE_LOST} event in LiveSync service for ${device.deviceInfo.identifier}. Will stop LiveSync operation for this device.`);
            for (const projectDir in this.processesInfo) {
                try {
                    const deviceDescriptors = this.getDeviceDescriptors(projectDir);
                    if (_.find(deviceDescriptors, d => d.identifier === device.deviceInfo.identifier)) {
                        yield this.stop(projectDir, [device.deviceInfo.identifier]);
                    }
                }
                catch (err) {
                    this.$logger.warn(`Unable to stop LiveSync operation for ${device.deviceInfo.identifier}.`, err);
                }
            }
        }));
    }
    syncInitialDataOnDevices(projectData, liveSyncInfo, deviceDescriptors) {
        return __awaiter(this, void 0, void 0, function* () {
            const deviceAction = (device) => __awaiter(this, void 0, void 0, function* () {
                const deviceDescriptor = _.find(deviceDescriptors, dd => dd.identifier === device.deviceInfo.identifier);
                const platformData = this.$platformsDataService.getPlatformData(device.deviceInfo.platform, projectData);
                const prepareData = this.$prepareDataService.getPrepareData(liveSyncInfo.projectDir, device.deviceInfo.platform, Object.assign({}, liveSyncInfo, { watch: !liveSyncInfo.skipWatcher, nativePrepare: { skipNativePrepare: !!deviceDescriptor.skipNativePrepare } }));
                const buildData = this.$buildDataService.getBuildData(projectData.projectDir, device.deviceInfo.platform, Object.assign({}, liveSyncInfo, { outputPath: deviceDescriptor.outputPath }));
                const prepareResultData = yield this.$prepareController.prepare(prepareData);
                try {
                    let packageFilePath = null;
                    const shouldBuild = prepareResultData.hasNativeChanges || (yield this.$buildController.shouldBuild(buildData));
                    if (shouldBuild) {
                        packageFilePath = yield deviceDescriptor.buildAction();
                    }
                    else {
                        yield this.$analyticsService.trackEventActionInGoogleAnalytics({
                            action: "LiveSync",
                            device,
                            projectDir: projectData.projectDir
                        });
                    }
                    yield this.$deviceInstallAppService.installOnDeviceIfNeeded(device, buildData, packageFilePath);
                    const platformLiveSyncService = this.$liveSyncServiceResolver.resolveLiveSyncService(platformData.platformNameLowerCase);
                    const { force, useHotModuleReload, skipWatcher } = liveSyncInfo;
                    const liveSyncResultInfo = yield platformLiveSyncService.fullSync({ force, useHotModuleReload, projectData, device, watch: !skipWatcher, liveSyncDeviceInfo: deviceDescriptor });
                    const refreshInfo = yield this.$deviceRefreshAppService.refreshApplication(projectData, liveSyncResultInfo, deviceDescriptor);
                    this.$runEmitter.emitRunExecutedEvent(projectData, device, {
                        syncedFiles: liveSyncResultInfo.modifiedFilesData.map(m => m.getLocalPath()),
                        isFullSync: liveSyncResultInfo.isFullSync
                    });
                    if (liveSyncResultInfo && deviceDescriptor.debuggingEnabled) {
                        yield this.$deviceDebugAppService.enableDebugging(projectData, deviceDescriptor, refreshInfo);
                    }
                    this.$logger.info(`Successfully synced application ${liveSyncResultInfo.deviceAppData.appIdentifier} on device ${liveSyncResultInfo.deviceAppData.device.deviceInfo.identifier}.`);
                    this.$runEmitter.emitRunStartedEvent(projectData, device);
                }
                catch (err) {
                    this.$logger.warn(`Unable to apply changes on device: ${device.deviceInfo.identifier}. Error is: ${err.message}.`);
                    this.$runEmitter.emitRunErrorEvent(projectData, device, err);
                }
            });
            yield this.addActionToChain(projectData.projectDir, () => this.$devicesService.execute(deviceAction, (device) => _.some(deviceDescriptors, deviceDescriptor => deviceDescriptor.identifier === device.deviceInfo.identifier)));
        });
    }
    syncChangedDataOnDevices(data, projectData, liveSyncInfo, deviceDescriptors) {
        return __awaiter(this, void 0, void 0, function* () {
            const deviceAction = (device) => __awaiter(this, void 0, void 0, function* () {
                const deviceDescriptor = _.find(deviceDescriptors, dd => dd.identifier === device.deviceInfo.identifier);
                const platformData = this.$platformsDataService.getPlatformData(data.platform, projectData);
                const prepareData = this.$prepareDataService.getPrepareData(projectData.projectDir, data.platform, Object.assign({}, liveSyncInfo, { watch: !liveSyncInfo.skipWatcher }));
                const buildData = this.$buildDataService.getBuildData(projectData.projectDir, data.platform, Object.assign({}, liveSyncInfo, { outputPath: deviceDescriptor.outputPath }));
                try {
                    if (data.hasNativeChanges) {
                        yield this.$prepareNativePlatformService.prepareNativePlatform(platformData, projectData, prepareData);
                        yield this.$buildController.prepareAndBuild(buildData);
                    }
                    const isInHMRMode = liveSyncInfo.useHotModuleReload && data.hmrData && data.hmrData.hash;
                    if (isInHMRMode) {
                        this.$hmrStatusService.watchHmrStatus(device.deviceInfo.identifier, data.hmrData.hash);
                    }
                    const platformLiveSyncService = this.$liveSyncServiceResolver.resolveLiveSyncService(device.deviceInfo.platform);
                    const watchInfo = {
                        liveSyncDeviceInfo: deviceDescriptor,
                        projectData,
                        filesToRemove: [],
                        filesToSync: data.files,
                        isReinstalled: false,
                        hmrData: data.hmrData,
                        useHotModuleReload: liveSyncInfo.useHotModuleReload,
                        force: liveSyncInfo.force,
                        connectTimeout: 1000
                    };
                    let liveSyncResultInfo = yield platformLiveSyncService.liveSyncWatchAction(device, watchInfo);
                    yield this.refreshApplication(projectData, liveSyncResultInfo, deviceDescriptor);
                    if (!liveSyncResultInfo.didRecover && isInHMRMode) {
                        const status = yield this.$hmrStatusService.getHmrStatus(device.deviceInfo.identifier, data.hmrData.hash);
                        if (status === constants_1.HmrConstants.HMR_ERROR_STATUS) {
                            watchInfo.filesToSync = data.hmrData.fallbackFiles;
                            liveSyncResultInfo = yield platformLiveSyncService.liveSyncWatchAction(device, watchInfo);
                            liveSyncResultInfo.isFullSync = true;
                            yield this.refreshApplication(projectData, liveSyncResultInfo, deviceDescriptor);
                        }
                    }
                    this.$logger.info(`Successfully synced application ${liveSyncResultInfo.deviceAppData.appIdentifier} on device ${liveSyncResultInfo.deviceAppData.device.deviceInfo.identifier}.`);
                }
                catch (err) {
                    const allErrors = err.allErrors;
                    if (allErrors && _.isArray(allErrors)) {
                        for (const deviceError of allErrors) {
                            this.$logger.warn(`Unable to apply changes for device: ${deviceError.deviceIdentifier}. Error is: ${deviceError.message}.`);
                            this.$runEmitter.emitRunErrorEvent(projectData, device, deviceError);
                        }
                    }
                }
            });
            yield this.addActionToChain(projectData.projectDir, () => this.$devicesService.execute(deviceAction, (device) => {
                const liveSyncProcessInfo = this.processesInfo[projectData.projectDir];
                return (data.platform.toLowerCase() === device.deviceInfo.platform.toLowerCase()) && liveSyncProcessInfo && _.some(liveSyncProcessInfo.deviceDescriptors, deviceDescriptor => deviceDescriptor.identifier === device.deviceInfo.identifier);
            }));
        });
    }
    refreshApplication(projectData, liveSyncResultInfo, deviceDescriptor) {
        return __awaiter(this, void 0, void 0, function* () {
            const refreshInfo = yield this.$deviceRefreshAppService.refreshApplication(projectData, liveSyncResultInfo, deviceDescriptor);
            this.$runEmitter.emitRunExecutedEvent(projectData, liveSyncResultInfo.deviceAppData.device, {
                syncedFiles: liveSyncResultInfo.modifiedFilesData.map(m => m.getLocalPath()),
                isFullSync: liveSyncResultInfo.isFullSync
            });
            if (liveSyncResultInfo && deviceDescriptor.debuggingEnabled) {
                yield this.$deviceDebugAppService.enableDebugging(projectData, deviceDescriptor, refreshInfo);
            }
        });
    }
    addActionToChain(projectDir, action) {
        return __awaiter(this, void 0, void 0, function* () {
            const liveSyncInfo = this.processesInfo[projectDir];
            if (liveSyncInfo) {
                liveSyncInfo.actionsChain = liveSyncInfo.actionsChain.then(() => __awaiter(this, void 0, void 0, function* () {
                    if (!liveSyncInfo.isStopped) {
                        liveSyncInfo.currentSyncAction = action();
                        const res = yield liveSyncInfo.currentSyncAction;
                        return res;
                    }
                }));
                const result = yield liveSyncInfo.actionsChain;
                return result;
            }
        });
    }
    persistData(projectDir, deviceDescriptors, platforms) {
        this.processesInfo[projectDir] = this.processesInfo[projectDir] || Object.create(null);
        this.processesInfo[projectDir].actionsChain = this.processesInfo[projectDir].actionsChain || Promise.resolve();
        this.processesInfo[projectDir].currentSyncAction = this.processesInfo[projectDir].actionsChain;
        this.processesInfo[projectDir].isStopped = false;
        this.processesInfo[projectDir].platforms = platforms;
        const currentDeviceDescriptors = this.getDeviceDescriptors(projectDir);
        this.processesInfo[projectDir].deviceDescriptors = _.uniqBy(currentDeviceDescriptors.concat(deviceDescriptors), "identifier");
    }
}
__decorate([
    decorators_1.cache()
], RunController.prototype, "attachDeviceLostHandler", null);
exports.RunController = RunController;
$injector.register("runController", RunController);
