"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const constants_1 = require("../constants");
class RunEmitter extends events_1.EventEmitter {
    constructor($logger) {
        super();
        this.$logger = $logger;
    }
    emitRunStartedEvent(projectData, device) {
        this.emitCore(constants_1.RunOnDeviceEvents.runOnDeviceStarted, {
            projectDir: projectData.projectDir,
            deviceIdentifier: device.deviceInfo.identifier,
            applicationIdentifier: projectData.projectIdentifiers[device.deviceInfo.platform.toLowerCase()]
        });
    }
    emitRunNotificationEvent(projectData, device, notification) {
        this.emitCore(constants_1.RunOnDeviceEvents.runOnDeviceNotification, {
            projectDir: projectData.projectDir,
            deviceIdentifier: device.deviceInfo.identifier,
            applicationIdentifier: projectData.projectIdentifiers[device.deviceInfo.platform.toLowerCase()],
            notification
        });
    }
    emitRunErrorEvent(projectData, device, error) {
        this.emitCore(constants_1.RunOnDeviceEvents.runOnDeviceError, {
            projectDir: projectData.projectDir,
            deviceIdentifier: device.deviceInfo.identifier,
            applicationIdentifier: projectData.projectIdentifiers[device.deviceInfo.platform.toLowerCase()],
            error,
        });
    }
    emitRunExecutedEvent(projectData, device, options) {
        this.emitCore(constants_1.RunOnDeviceEvents.runOnDeviceExecuted, {
            projectDir: projectData.projectDir,
            deviceIdentifier: device.deviceInfo.identifier,
            applicationIdentifier: projectData.projectIdentifiers[device.deviceInfo.platform.toLowerCase()],
            syncedFiles: options.syncedFiles,
            isFullSync: options.isFullSync
        });
    }
    emitRunStoppedEvent(projectDir, deviceIdentifier) {
        this.emitCore(constants_1.RunOnDeviceEvents.runOnDeviceStopped, {
            projectDir,
            deviceIdentifier
        });
    }
    emitDebuggerAttachedEvent(debugInformation) {
        this.emit(constants_1.DEBUGGER_ATTACHED_EVENT_NAME, debugInformation);
    }
    emitDebuggerDetachedEvent(device) {
        const deviceIdentifier = device.deviceInfo.identifier;
        this.emit(constants_1.DEBUGGER_DETACHED_EVENT_NAME, { deviceIdentifier });
    }
    emitUserInteractionNeededEvent(projectData, device, deviceDescriptor) {
        const deviceIdentifier = device.deviceInfo.identifier;
        const attachDebuggerOptions = {
            platform: device.deviceInfo.platform,
            isEmulator: device.isEmulator,
            projectDir: projectData.projectDir,
            deviceIdentifier,
            debugOptions: deviceDescriptor.debugOptions,
            outputPath: deviceDescriptor.outputPath
        };
        this.emit(constants_1.USER_INTERACTION_NEEDED_EVENT_NAME, attachDebuggerOptions);
    }
    emitCore(event, data) {
        this.$logger.trace(`Will emit event ${event} with data`, data);
        this.emit(event, data);
    }
}
exports.RunEmitter = RunEmitter;
$injector.register("runEmitter", RunEmitter);
