"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const decorators_1 = require("../../common/decorators");
class DeviceRefreshAppService {
    constructor($liveSyncServiceResolver, $logger, $runEmitter) {
        this.$liveSyncServiceResolver = $liveSyncServiceResolver;
        this.$logger = $logger;
        this.$runEmitter = $runEmitter;
    }
    refreshApplication(projectData, liveSyncResultInfo, deviceDescriptor, settings) {
        return __awaiter(this, void 0, void 0, function* () {
            if (deviceDescriptor && deviceDescriptor.debuggingEnabled) {
                liveSyncResultInfo.waitForDebugger = deviceDescriptor.debugOptions && deviceDescriptor.debugOptions.debugBrk;
            }
            const result = { didRestart: false };
            const platform = liveSyncResultInfo.deviceAppData.platform;
            const applicationIdentifier = projectData.projectIdentifiers[platform.toLowerCase()];
            const platformLiveSyncService = this.$liveSyncServiceResolver.resolveLiveSyncService(platform);
            try {
                let shouldRestart = yield platformLiveSyncService.shouldRestart(projectData, liveSyncResultInfo);
                if (!shouldRestart) {
                    shouldRestart = !(yield platformLiveSyncService.tryRefreshApplication(projectData, liveSyncResultInfo));
                }
                if (shouldRestart) {
                    this.$runEmitter.emitDebuggerDetachedEvent(liveSyncResultInfo.deviceAppData.device);
                    yield platformLiveSyncService.restartApplication(projectData, liveSyncResultInfo);
                    result.didRestart = true;
                }
            }
            catch (err) {
                this.$logger.info(`Error while trying to start application ${applicationIdentifier} on device ${liveSyncResultInfo.deviceAppData.device.deviceInfo.identifier}. Error is: ${err.message || err}`);
                const msg = `Unable to start application ${applicationIdentifier} on device ${liveSyncResultInfo.deviceAppData.device.deviceInfo.identifier}. Try starting it manually.`;
                this.$logger.warn(msg);
                if (!settings || !settings.shouldSkipEmitLiveSyncNotification) {
                    this.$runEmitter.emitRunNotificationEvent(projectData, liveSyncResultInfo.deviceAppData.device, msg);
                }
                if (settings && settings.shouldCheckDeveloperDiscImage && (err.message || err) === "Could not find developer disk image") {
                    this.$runEmitter.emitUserInteractionNeededEvent(projectData, liveSyncResultInfo.deviceAppData.device, deviceDescriptor);
                }
            }
            return result;
        });
    }
}
__decorate([
    decorators_1.performanceLog()
], DeviceRefreshAppService.prototype, "refreshApplication", null);
exports.DeviceRefreshAppService = DeviceRefreshAppService;
$injector.register("deviceRefreshAppService", DeviceRefreshAppService);
