"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const constants_1 = require("../constants");
const helpers_1 = require("../common/helpers");
const prepareInfoFileName = ".nsprepareinfo";
class ProjectChangesInfo {
    get hasChanges() {
        return this.packageChanged ||
            this.appResourcesChanged ||
            this.modulesChanged ||
            this.configChanged ||
            this.signingChanged;
    }
    get changesRequireBuild() {
        return this.packageChanged ||
            this.appResourcesChanged ||
            this.nativeChanged;
    }
    get changesRequirePrepare() {
        return this.appResourcesChanged ||
            this.signingChanged;
    }
}
class ProjectChangesService {
    constructor($devicePlatformsConstants, $fs, $logger, $hooksService) {
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.$fs = $fs;
        this.$logger = $logger;
        this.$hooksService = $hooksService;
        this._newFiles = 0;
    }
    get currentChanges() {
        return this._changesInfo;
    }
    checkForChanges(platformData, projectData, prepareData) {
        return __awaiter(this, void 0, void 0, function* () {
            this._changesInfo = new ProjectChangesInfo();
            const isNewPrepareInfo = yield this.ensurePrepareInfo(platformData, projectData, prepareData);
            if (!isNewPrepareInfo) {
                this._newFiles = 0;
                this._changesInfo.packageChanged = this.isProjectFileChanged(projectData.projectDir, platformData);
                const platformResourcesDir = path.join(projectData.appResourcesDirectoryPath, platformData.normalizedPlatformName);
                this._changesInfo.appResourcesChanged = this.containsNewerFiles(platformResourcesDir, null, projectData);
                this._changesInfo.nativeChanged = this.containsNewerFiles(path.join(projectData.projectDir, constants_1.NODE_MODULES_FOLDER_NAME), path.join(projectData.projectDir, constants_1.NODE_MODULES_FOLDER_NAME, "tns-ios-inspector"), projectData, this.fileChangeRequiresBuild);
                this.$logger.trace(`Set nativeChanged to ${this._changesInfo.nativeChanged}.`);
                if (this._newFiles > 0 || this._changesInfo.nativeChanged) {
                    this.$logger.trace(`Setting modulesChanged to true, newFiles: ${this._newFiles}, nativeChanged: ${this._changesInfo.nativeChanged}`);
                    this._changesInfo.modulesChanged = true;
                }
                if (platformData.platformNameLowerCase === this.$devicePlatformsConstants.iOS.toLowerCase()) {
                    this._changesInfo.configChanged = this.filesChanged([path.join(platformResourcesDir, platformData.configurationFileName),
                        path.join(platformResourcesDir, "LaunchScreen.storyboard"),
                        path.join(platformResourcesDir, constants_1.BUILD_XCCONFIG_FILE_NAME)
                    ]);
                }
                else {
                    this._changesInfo.configChanged = this.filesChanged([
                        path.join(platformResourcesDir, platformData.configurationFileName),
                        path.join(platformResourcesDir, constants_1.APP_GRADLE_FILE_NAME)
                    ]);
                }
                this.$logger.trace(`Set value of configChanged to ${this._changesInfo.configChanged}`);
            }
            if (!prepareData.nativePrepare || !prepareData.nativePrepare.skipNativePrepare) {
                yield platformData.platformProjectService.checkForChanges(this._changesInfo, prepareData, projectData);
            }
            if (prepareData.release !== this._prepareInfo.release) {
                this.$logger.trace(`Setting all setting to true. Current options are: `, prepareData, " old prepare info is: ", this._prepareInfo);
                this._changesInfo.appResourcesChanged = true;
                this._changesInfo.modulesChanged = true;
                this._changesInfo.configChanged = true;
                this._prepareInfo.release = prepareData.release;
            }
            if (this._changesInfo.packageChanged) {
                this.$logger.trace("Set modulesChanged to true as packageChanged is true");
                this._changesInfo.modulesChanged = true;
            }
            if (this._changesInfo.modulesChanged || this._changesInfo.appResourcesChanged) {
                this.$logger.trace(`Set configChanged to true, current value of moduleChanged is: ${this._changesInfo.modulesChanged}, appResourcesChanged is: ${this._changesInfo.appResourcesChanged}`);
                this._changesInfo.configChanged = true;
            }
            if (this._changesInfo.hasChanges) {
                this._prepareInfo.changesRequireBuild = this._changesInfo.changesRequireBuild;
                this._prepareInfo.time = new Date().toString();
                if (this._prepareInfo.changesRequireBuild) {
                    this._prepareInfo.changesRequireBuildTime = this._prepareInfo.time;
                }
                this._prepareInfo.projectFileHash = this.getProjectFileStrippedHash(projectData.projectDir, platformData);
            }
            this._changesInfo.nativePlatformStatus = this._prepareInfo.nativePlatformStatus;
            this.$logger.trace("checkForChanges returns", this._changesInfo);
            return this._changesInfo;
        });
    }
    getPrepareInfoFilePath(platformData) {
        const prepareInfoFilePath = path.join(platformData.projectRoot, prepareInfoFileName);
        return prepareInfoFilePath;
    }
    getPrepareInfo(platformData) {
        const prepareInfoFilePath = this.getPrepareInfoFilePath(platformData);
        let prepareInfo = null;
        if (this.$fs.exists(prepareInfoFilePath)) {
            try {
                prepareInfo = this.$fs.readJson(prepareInfoFilePath);
            }
            catch (e) {
                prepareInfo = null;
            }
        }
        return prepareInfo;
    }
    savePrepareInfo(platformData) {
        const prepareInfoFilePath = this.getPrepareInfoFilePath(platformData);
        this.$fs.writeJson(prepareInfoFilePath, this._prepareInfo);
    }
    setNativePlatformStatus(platformData, addedPlatform) {
        this._prepareInfo = this._prepareInfo || this.getPrepareInfo(platformData);
        if (this._prepareInfo && addedPlatform.nativePlatformStatus === "3") {
            this._prepareInfo.nativePlatformStatus = addedPlatform.nativePlatformStatus;
        }
        else {
            this._prepareInfo = {
                nativePlatformStatus: addedPlatform.nativePlatformStatus
            };
        }
        this.savePrepareInfo(platformData);
    }
    ensurePrepareInfo(platformData, projectData, prepareData) {
        return __awaiter(this, void 0, void 0, function* () {
            this._prepareInfo = this.getPrepareInfo(platformData);
            if (this._prepareInfo) {
                const prepareInfoFile = path.join(platformData.projectRoot, prepareInfoFileName);
                this._outputProjectMtime = this.$fs.getFsStats(prepareInfoFile).mtime.getTime();
                this._outputProjectCTime = this.$fs.getFsStats(prepareInfoFile).ctime.getTime();
                return false;
            }
            const nativePlatformStatus = (!prepareData.nativePrepare || !prepareData.nativePrepare.skipNativePrepare) ?
                "2" : "1";
            this._prepareInfo = {
                time: "",
                nativePlatformStatus,
                release: prepareData.release,
                changesRequireBuild: true,
                projectFileHash: this.getProjectFileStrippedHash(projectData.projectDir, platformData),
                changesRequireBuildTime: null
            };
            this._outputProjectMtime = 0;
            this._outputProjectCTime = 0;
            this._changesInfo = this._changesInfo || new ProjectChangesInfo();
            this._changesInfo.appResourcesChanged = true;
            this._changesInfo.modulesChanged = true;
            this._changesInfo.configChanged = true;
            return true;
        });
    }
    getProjectFileStrippedHash(projectDir, platformData) {
        const projectFilePath = path.join(projectDir, constants_1.PACKAGE_JSON_FILE_NAME);
        const projectFileContents = this.$fs.readJson(projectFilePath);
        _(this.$devicePlatformsConstants)
            .keys()
            .map(k => k.toLowerCase())
            .difference([platformData.platformNameLowerCase])
            .each(otherPlatform => {
            delete projectFileContents.nativescript[`tns-${otherPlatform}`];
        });
        return helpers_1.getHash(JSON.stringify(projectFileContents));
    }
    isProjectFileChanged(projectDir, platformData) {
        const projectFileStrippedContentsHash = this.getProjectFileStrippedHash(projectDir, platformData);
        const prepareInfo = this.getPrepareInfo(platformData);
        return projectFileStrippedContentsHash !== prepareInfo.projectFileHash;
    }
    filesChanged(files) {
        for (const file of files) {
            if (this.$fs.exists(file)) {
                const fileStats = this.$fs.getFsStats(file);
                if (fileStats.mtime.getTime() >= this._outputProjectMtime || fileStats.ctime.getTime() >= this._outputProjectCTime) {
                    return true;
                }
            }
        }
        return false;
    }
    containsNewerFiles(dir, skipDir, projectData, processFunc) {
        const dirName = path.basename(dir);
        this.$logger.trace(`containsNewerFiles will check ${dir}`);
        if (_.startsWith(dirName, '.')) {
            this.$logger.trace(`containsNewerFiles returns false for ${dir} as its name starts with dot (.) .`);
            return false;
        }
        const dirFileStat = this.$fs.getFsStats(dir);
        if (this.isFileModified(dirFileStat, dir)) {
            this.$logger.trace(`containsNewerFiles returns true for ${dir} as the dir itself has been modified.`);
            return true;
        }
        const files = this.$fs.readDirectory(dir);
        for (const file of files) {
            const filePath = path.join(dir, file);
            if (filePath === skipDir) {
                continue;
            }
            const fileStats = this.$fs.getFsStats(filePath);
            const changed = this.isFileModified(fileStats, filePath);
            if (changed) {
                this.$logger.trace(`File ${filePath} has been changed.`);
                if (processFunc) {
                    this._newFiles++;
                    this.$logger.trace(`Incremented the newFiles counter. Current value is ${this._newFiles}`);
                    const filePathRelative = path.relative(projectData.projectDir, filePath);
                    if (processFunc.call(this, filePathRelative, projectData)) {
                        this.$logger.trace(`containsNewerFiles returns true for ${dir}. The modified file is ${filePath}`);
                        return true;
                    }
                }
                else {
                    this.$logger.trace(`containsNewerFiles returns true for ${dir}. The modified file is ${filePath}`);
                    return true;
                }
            }
            if (fileStats.isDirectory()) {
                if (this.containsNewerFiles(filePath, skipDir, projectData, processFunc)) {
                    this.$logger.trace(`containsNewerFiles returns true for ${dir}.`);
                    return true;
                }
            }
        }
        this.$logger.trace(`containsNewerFiles returns false for ${dir}.`);
        return false;
    }
    isFileModified(filePathStat, filePath) {
        let changed = filePathStat.mtime.getTime() >= this._outputProjectMtime ||
            filePathStat.ctime.getTime() >= this._outputProjectCTime;
        if (!changed) {
            const lFileStats = this.$fs.getLsStats(filePath);
            changed = lFileStats.mtime.getTime() >= this._outputProjectMtime ||
                lFileStats.ctime.getTime() >= this._outputProjectCTime;
        }
        return changed;
    }
    fileChangeRequiresBuild(file, projectData) {
        if (path.basename(file) === constants_1.PACKAGE_JSON_FILE_NAME) {
            return true;
        }
        const projectDir = projectData.projectDir;
        if (_.startsWith(path.join(projectDir, file), projectData.appResourcesDirectoryPath)) {
            return true;
        }
        if (_.startsWith(file, constants_1.NODE_MODULES_FOLDER_NAME)) {
            let filePath = file;
            while (filePath !== constants_1.NODE_MODULES_FOLDER_NAME) {
                filePath = path.dirname(filePath);
                const fullFilePath = path.join(projectDir, path.join(filePath, constants_1.PACKAGE_JSON_FILE_NAME));
                if (this.$fs.exists(fullFilePath)) {
                    const json = this.$fs.readJson(fullFilePath);
                    if (json["nativescript"] && _.startsWith(file, path.join(filePath, "platforms"))) {
                        return true;
                    }
                }
            }
        }
        return false;
    }
}
__decorate([
    helpers_1.hook("checkForChanges")
], ProjectChangesService.prototype, "checkForChanges", null);
exports.ProjectChangesService = ProjectChangesService;
$injector.register("projectChangesService", ProjectChangesService);
